<?php

namespace MyApp\Core;

use ReflectionClass;
use ReflectionProperty;

class Model
{
    /**
     * @var array
     */
    private $internalTableMap;

    public function __construct(?array $tableMap = null)
    {
        if (!is_array($tableMap)) {
            $this->setInternalTableMap();
        }
    }

    private function setInternalTableMap()
    {
        /* @var \App $app */
        global $app;

        $tableMap = ['type' => 'bd'];

        $refClass = new ReflectionClass(get_called_class());
        $tags = $app->parseTagsFromComment($refClass->getDocComment(),
            'TableName');
        if ($tags) {
            if (array_key_exists('TableName', $tags)) {
                $tableMap['name'] = $tags['TableName'];
            }

            $properties = $refClass->getProperties(ReflectionProperty::IS_PRIVATE);
            foreach ($properties as $property) {
                $propertyTags = $app->parseTagsFromComment($property->getDocComment(),
                    'ColumnName|ColumnOption|ColumnType');

                if ($propertyTags) {
                    if (array_key_exists('ColumnName', $propertyTags)) {
                        $tableMap['fields'][$property->name] = $propertyTags['ColumnName'];
                        if (array_key_exists('ColumnOption', $propertyTags)) {
                            if ('id' === $propertyTags['ColumnOption']) {
                                $tableMap['id'] = $property->name;
                            }
                        }
                        if (array_key_exists('ColumnType', $propertyTags)) {
                            $tableMap['types'][$propertyTags['ColumnName']] = $propertyTags['ColumnType'];
                        }
                    }
                }
            }
            $this->internalTableMap = array_key_exists('id',
                $tableMap) ? $tableMap : ['type' => 'other'];
        }
    }

    public function find($id)
    {
    }

    /**
     * @return array
     */
    public function getInternalTableMap(): ?array
    {
        return $this->internalTableMap;
    }
}
